/*
  Stockfish, a UCI chess playing engine derived from Glaurung 2.1
  Copyright (C) 2004-2008 Tord Romstad (Glaurung author)
  Copyright (C) 2008-2015 Marco Costalba, Joona Kiiski, Tord Romstad
  Copyright (C) 2015-2019 Marco Costalba, Joona Kiiski, Gary Linscott, Tord Romstad

  Stockfish is free software: you can redistribute it and/or modify
  it under the terms of the GNU General Public License as published by
  the Free Software Foundation, either version 3 of the License, or
  (at your option) any later version.

  Stockfish is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU General Public License for more details.

  You should have received a copy of the GNU General Public License
  along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/

#include <iostream>
#include <thread>

#include "bitboard.h"
#include "position.h"
#include "search.h"
#include "thread.h"
#include "tt.h"
#include "uci.h"
#include "syzygy/tbprobe.h"

#include <grpc/grpc.h>
#include <grpc++/server.h>
#include <grpc++/server_builder.h>
#include "hashprobe.grpc.pb.h"

using grpc::Server;
using grpc::ServerBuilder;
using grpc::ServerContext;
using grpc::Status;
using grpc::StatusCode;

class HashProbeImpl final : public HashProbe::Service {
public:
	Status Probe(ServerContext* context,
	             const HashProbeRequest* request,
	             HashProbeResponse *response) {
		Position pos(request->fen(), /*isChess960=*/false, Threads.main());
		if (!pos.pos_is_ok()) {
			return Status(StatusCode::INVALID_ARGUMENT, "Invalid FEN");
		}
		bool found;
		TTEntry *entry = TT.probe(pos.key(), found);
		response->set_found(found);
		if (found) {
			response->set_move(entry->move());
			response->set_value(entry->value());
			response->set_eval(entry->eval());
			response->set_depth(entry->depth());
			response->set_bound(HashProbeResponse::ValueBound(entry->bound()));
		}
		return Status::OK;
	}
};
	
void rpc_thread()
{
	std::string server_address("0.0.0.0:50051");
	HashProbeImpl service;

	ServerBuilder builder;
	builder.AddListeningPort(server_address, grpc::InsecureServerCredentials());
	builder.RegisterService(&service);
	std::unique_ptr<Server> server(builder.BuildAndStart());
	std::cout << "Server listening on " << server_address << std::endl;
	server->Wait();
}

namespace PSQT {
  void init();
}

int main(int argc, char* argv[]) {

  std::cout << engine_info() << std::endl;

  UCI::init(Options);
  PSQT::init();
  Bitboards::init();
  Position::init();
  Bitbases::init();
  Search::init();
  Pawns::init();
  Threads.set(Options["Threads"]);
  Search::clear(); // After threads are up

  std::thread(&rpc_thread).detach();

  UCI::loop(argc, argv);

  Threads.set(0);
  return 0;
}
